<#
	.SYNOPSIS
		Query your ITOps Board license expiration date relative to now

	.DESCRIPTION
		This script is able to query the ITOps Board API to keep track of its license expiration. The script is designed to be used with PRTG Network Monitor (https://www.paessler.com/prtg). IMPORTANT: Remember to configure the appropriate limits in the channel settings for the expiration to get alert.

	.PARAMETER TargetName
		The Address (FQDN Preferred) of the ITOps Board Server, e.g.: board.example.org

	.PARAMETER TargetPort
		The TCP Port on which to contact the ITOps Board Server. (Default: 59212)

	.PARAMETER UserName
		The account (mind the domain.tld\username format) used for the query, e.g.: example.org\jdoe

	.PARAMETER Password
		The user's password

	.PARAMETER Timeout
		The number of seconds to wait for the HTTP query to complete (Default: 30)

	.PARAMETER TrustAllCerts
		Switch parameter to force Powershell to connect to endpoints with SSL Issues (Self-Signed cert, wrong cert, etc) which introduces security risks (Default: Disabled)

	.EXAMPLE
		Running the script interactively to test it:
		PS> .\Get-ITOpsBoardLicenseExpiry.ps1 -TargetName board.example.org -TargetPort 443 -UserName example.org\svc-account -Password myp4ssw0rd -Timeout 30

	.EXAMPLE
		For running the script in PRTG, use PRTG's placeholders to make the setup more robust (and optionally ignore SSL Certificate Errors with -TrustAllCerts)
		Parameters(in the Sensor's settings): -TargetName '%host' -TargetPort 443 -UserName '%windowsdomain\%windowsuser' -Password '%windowspassword' -Timeout 30 -TrustAllCerts

	.NOTES
		Version: 0.92.20220324
		Author: Luciano Lingnau, Paessler AG (https://www.paessler.com/)

	.LINK
		https://kb.paessler.com/en/topic/90443
#>

#Requires -Version 3.0

param(
	#[Parameter(Mandatory=$true)]
    [string]$TargetName,
    [int]$TargetPort = "59212",
	#[Parameter(Mandatory=$true)]
	[string]$UserName,
	#[Parameter(Mandatory=$true)]
	[string]$Password,
	[int]$Timeout = "30",
	[switch]$TrustAllCerts = $true
)

#Create a pscredential which will be needed for the query later
$PasswordSec = ConvertTo-SecureString $Password -AsPlainText -Force
$Credential = New-Object -TypeName System.Management.Automation.PSCredential -ArgumentList $UserName, $PasswordSec

[string]$sensorMessage = ''
$errorFlag = $false
$results = @()

#Avoid issues with weird SSL Certificates (Self-signed, expired, etc)
#Should only be used when a trusted certificate is not an option
if ($TrustAllCerts){
add-type @"
    using System.Net;
    using System.Security.Cryptography.X509Certificates;
    public class TrustAllCertsPolicy : ICertificatePolicy {
        public bool CheckValidationResult(
            ServicePoint srvPoint, X509Certificate certificate,
            WebRequest request, int certificateProblem) {
            return true;
        }
    }
"@
[System.Net.ServicePointManager]::CertificatePolicy = New-Object TrustAllCertsPolicy
}

try {
	#Stopwatch used to time the lenghtiest part of the execution (long execution times could be an issue)
	$exectime = [System.Diagnostics.Stopwatch]::StartNew()

	#Imports ConvertFrom-Json2.ps1 from the subfolder named "include"
	#This is used to more easily parse the json
	#For credits please check the file itself
	. "$PSScriptRoot\include\ConvertFrom-Json2.ps1"

	#Actual query for the license summary of itopsboard
	$a = Invoke-Webrequest -Uri "https://$($TargetName):$($TargetPort)/api/licenses/summary" -Credential $Credential -TimeoutSec $Timeout -UseBasicParsing -ErrorAction SilentlyContinue

	#Stop Stopwatch used to time the lenghtiest part of the execution
	$exectime.Stop()

	$b =  $a.Content | ConvertFrom-Json2

	#Calculate the number of days until the expiration date (as of today)
	$Start = Get-Date
	$Span = New-TimeSpan -Start $start -End $b.expirationDate
	$Daysremain = [math]::Round($span.TotalDays,0)

	#Parse the results in a PRTG Friendly way

	$results += @{
		Channel = 'Days to Expiration'
		Value = $Daysremain
		Unit = 'Custom'
		CustomUnit = 'Days'
	}
	$results += @{
		Channel = 'Execution Time'
		Value = $exectime.ElapsedMilliseconds
		Unit = 'TimeResponse'
	}

    $sensorMessage = "License for $($b.serverName) expires on $($b.expirationDate)"

} catch {
	
	#When no license is found the ITOpsboard will throw a 400 error with a specific message (Thanks to https://kb.paessler.com/users/ct)
	#https://kb.paessler.com/en/topic/90443-can-prtg-monitor-the-license-expiration-of-itops-board#reply-314300
    if ($_.Exception.Response.StatusCode.Value__ -eq 400 -And $_.toString() -eq '{"message":"No License was Found. Please add a new license."}' ){

		$sensorMessage = "No valid license found on $($TargetName)"
        $errorFlag  = $true

	}else{

		$errorFlag  = $true
		$sensorMessage = "Exception (Line $($_.InvocationInfo.ScriptLineNumber)): $($_.Exception.Message)"

	}
}

$sensorResult = '' | Select-Object prtg
if ($errorFlag) {
	$sensorResult.prtg = '' | Select-Object error
	$sensorResult.prtg.error = 1
} else {
	$sensorResult.prtg = '' | Select-Object result
	$sensorResult.prtg.result = $results
}
if ($sensorMessage) {
	$sensorResult.prtg | Add-Member -MemberType NoteProperty -Name 'text' -Value $null
	$sensorResult.prtg.text = $sensorMessage
}
$sensorResultJson = $sensorResult | ConvertTo-Json -Depth 5
Write-Output $sensorResultJson